%*******************************************************************************
% DISCLAIMER OF LIABILITY
%
% This text/file contains proprietary, confidential
% information of Xilinx, Inc., is distributed under license
% from Xilinx, Inc., and may be used, copied and/or
% disclosed only pursuant to the terms of a valid license
% agreement with Xilinx, Inc. Xilinx hereby grants you a 
% license to use this text/file solely for design, simulation, 
% implementation and creation of design files limited 
% to Xilinx devices or technologies. Use with non-Xilinx 
% devices or technologies is expressly prohibited and 
% immediately terminates your license unless covered by
% a separate agreement.
%
% Xilinx is providing this design, code, or information 
% "as-is" solely for use in developing programs and 
% solutions for Xilinx devices, with no obligation on the 
% part of Xilinx to provide support. By providing this design, 
% code, or information as one possible implementation of 
% this feature, application or standard, Xilinx is making no 
% representation that this implementation is free from any 
% claims of infringement. You are responsible for 
% obtaining any rights you may require for your implementation. 
% Xilinx expressly disclaims any warranty whatsoever with 
% respect to the adequacy of the implementation, including 
% but not limited to any warranties or representations that this
% implementation is free from claims of infringement, implied 
% warranties of merchantability or fitness for a particular 
% purpose.
%
% Xilinx products are not intended for use in life support
% appliances, devices, or systems. Use in such applications is
% expressly prohibited.
%
% Any modifications that are made to the Source Code are 
% done at the users sole risk and will be unsupported.
%
%
% Copyright (c) 2007 Xilinx, Inc. All rights reserved.
%
% This copyright and support notice must be retained as part 
% of this text at all times. 
%
%*******************************************************************************

%*******************************************************************************
% cdma_ddc_filter.m
% This function generates filter coeffs for all three stages of the CDMA DDC.
% The first two 5x decimator FIRs are lowpasses with the coeffs calculated 
% with Parks-McClellan.  Coeffs are rounded to the desired bitwidth which 
% should match the storage size in the FIR hardware.
% The last stage is a shaping FIR.  The shaping coeffs are given
% in the CDMA spec.
% 
% Author:  Helen Tarn, Xilinx, 2007
% Updated: Kevin Neilson, Xilinx
%*******************************************************************************
function [h1, h2, h3] = cdma_ddc_filter(Quantize, Nfft, ShowPlots)

if (nargin<3)
    ShowPlots = 0;
end

if (nargin<2)
    Nfft = 32768;
end

% Create Quantizer Object used to round all coeffs
filt_quant=quantizer('fixed','floor','wrap',[20,19]);

Fs = 61.44;

%***********************************************************************
% Scaling cascaded filters is somewhat of an art.  There is a tradeoff  
% involved.  Each filter can be scaled so that the the sum of the       
% absolute values of any polyphase is less than 1; this ensures that    
% there will never be overflows that cause wrapping.  However this      
% conservative scaling will also ensure that the utilized dynamic range 
% of the final outputs will be very small.  (Note too that the three-   
% channel mixer also requires scaling by 1/3 to prevent overflows.)     
% It is desirable to use as much of the dynamic range of the output     
% as possible while having a minimum of overflows.  This was done by    
% first scaling all filters for no overflow potential and then          
% increasing each one until overflow/wrapping occured in the simulation.
% The scale_tweak vector here contains these tweak values.  A 1 tweak 
% value corresponds to the minimum value for which no overflow will ever 
% occur.  The user may change the tweak values as desired.                                          
%                                                                       
% A FIR filter obtains its maximum output when the input delay line     
% contains +1 for each postitive coefficient and -1 for each negative   
% coefficient.  That is why the maximum value can be computed by        
% taking the sum of the absolute values of the coeffs of any polyphase. 
% The statistical possibility of overflow increases as the number of    
% taps in the filter increases and increases as the rolloff factor      
% decreases, as the coeff size decreases more slowly from the center    
% tap.                                                                  
%                                                                       
% Each successive FIR in a cascaded chain is less likely to overflow.   
%                                                                       
% To increase the dynamic range further, CFR (Crest Factor Reduction)   
% may be used following the mixer output.  This usually involves        
% a distortion of the signal for the statistically-less-probable        
% signals of high amplitude.                                            
%***********************************************************************
scale_tweak = [1.0 2.0 2.0];

% Create Quantizer Object used to round all coeffs
filt_quant=quantizer('fixed','floor','wrap',[20,19]);

%****************************************************************
% Shaping filter coefficients
% These coeffs are given in the CDMA spec.
%****************************************************************
h = [-0.025288315, -0.034167931, -0.035752323, -0.016733702,  0.021602514,...
      0.064938487,  0.091002137,  0.081894974,  0.037071157, -0.021998074,...
     -0.060716277, -0.051178658,  0.007874526,  0.084368728,  0.126869306,...
      0.094528345, -0.012839661, -0.143477028, -0.211829088, -0.140513128,...
      0.094601918,  0.441387140,  0.785875640,  1.0];
h = [h,fliplr(h)]; % Make filter symmetric with odd number of taps

% First Decimation Filter 
Hd = dec1;
h1 = Hd.Numerator;
H1 = 20*log10(eps + abs(fftshift(fft(h1, Nfft))));

% Scale coeffs to prevent overflows
h1 = scale_coeffs(h1,1).*scale_tweak(1);

if Quantize
    h1 = quantize(filt_quant,h1);
    H1 = 20*log10(eps + abs(fftshift(fft(h1, Nfft))));
end

if ShowPlots
    figure
    plot(linspace(-Fs/2,Fs/2,Nfft), H1-max(H1), 'b');
    axis([0,Fs/2,-100,0])
    hold on; grid on; zoom on;
    title('Magnitude Response of First Filter (Decimation-by-5)');
    xlabel('Frequency (MHz)')
    ylabel('dB')
    legend('Filter Response');
end

% Second Decimation Filter
Fs = Fs/5;
Hd = dec2;
h2 = Hd.Numerator;
H2 = 20*log10(eps + abs(fftshift(fft(h2, Nfft))));

% Scale coeffs to prevent overflows
h2 = scale_coeffs(h2,1).*scale_tweak(2);

if Quantize
    h2 = quantize(filt_quant,h2);
    H2 = 20*log10(eps + abs(fftshift(fft(h2, Nfft))));
end

if ShowPlots
    figure
    plot(linspace(-Fs/2,Fs/2,Nfft), H2-max(H2), 'b');
    hold on; grid on; zoom on;
    axis([0,Fs/2,-100,0])
    title('Magnitude Response of Second Filter (Decimation-by-5)');
    xlabel('Frequency (MHz)')
    ylabel('dB')
    legend('Filter Response');
end

% Third Decimation Filter
Fs = Fs/5;

Hd = channel_fir;
h31 = Hd.Numerator;
h32 = resample(h, 2, 1);
h32 = resample([0, h32(1:end-1)], 1, 4);
h3 = conv(h31, h32(2:end));
h3 = h3(2:end-1);
H3 = 20*log10(eps + abs(fftshift(fft(h3, Nfft))));

% Scale coeffs to prevent overflows
h3 = scale_coeffs(h3,1).*scale_tweak(3);

if Quantize
    h3 = quantize(filt_quant,h3);
    H3 = 20*log10(eps + abs(fftshift(fft(h3, Nfft))));
end

if ShowPlots
    figure
    plot(linspace(-Fs/2,Fs/2,Nfft), H3-max(H3), 'b');
    hold on; grid on; zoom on;
    axis([0,Fs/2,-100,0])
    title('Magnitude Response of Third Filter (Single Rate Channel Filter)');
    xlabel('Frequency (MHz)')
    ylabel('dB')
    legend('Filter Response');
end