%*******************************************************************************
% DISCLAIMER OF LIABILITY
%
% This text/file contains proprietary, confidential
% information of Xilinx, Inc., is distributed under license
% from Xilinx, Inc., and may be used, copied and/or
% disclosed only pursuant to the terms of a valid license
% agreement with Xilinx, Inc. Xilinx hereby grants you a 
% license to use this text/file solely for design, simulation, 
% implementation and creation of design files limited 
% to Xilinx devices or technologies. Use with non-Xilinx 
% devices or technologies is expressly prohibited and 
% immediately terminates your license unless covered by
% a separate agreement.
%
% Xilinx is providing this design, code, or information 
% "as-is" solely for use in developing programs and 
% solutions for Xilinx devices, with no obligation on the 
% part of Xilinx to provide support. By providing this design, 
% code, or information as one possible implementation of 
% this feature, application or standard, Xilinx is making no 
% representation that this implementation is free from any 
% claims of infringement. You are responsible for 
% obtaining any rights you may require for your implementation. 
% Xilinx expressly disclaims any warranty whatsoever with 
% respect to the adequacy of the implementation, including 
% but not limited to any warranties or representations that this
% implementation is free from claims of infringement, implied 
% warranties of merchantability or fitness for a particular 
% purpose.
%
% Xilinx products are not intended for use in life support
% appliances, devices, or systems. Use in such applications is
% expressly prohibited.
%
% Any modifications that are made to the Source Code are 
% done at the users sole risk and will be unsupported.
%
%
% Copyright (c) 2007 Xilinx, Inc. All rights reserved.
%
% This copyright and support notice must be retained as part 
% of this text at all times. 
%
%*******************************************************************************

%*******************************************************************************
% cdma_ddc_model.m
% 
% This is a model for the CDMA 3-channel DDC (Digital DownConverter).
% The model attempts to replicate the hardware as much as possible, by 
% quantizing appropriately.
% 
% Original:  Helen Tarn, Xilinx
% Update:    Kevin Neilson, Xilinx
%*******************************************************************************

%****************************************************************
% Set parameters
%****************************************************************
Quantize = 1;   % double (0) or quantized (1) coefficients and data 
Nfft = 1024;    % FFT size for spectral plot
ShowPlots = 0;  % Show filter impluse response plots
NUM_CH = 3;     % Define number of channels


Fc = [-1.25;0;1.25]; % Column vector of center mixing frequencies (MHz)
Fs = 61.44;          % Sampling Frequency
t = [0:1:999];       % Time vector

%****************************************************************
% Load complex input vector
%****************************************************************
if (exist('composite_iq.mat'))
  load('composite_iq.mat');
else
  error('File composite_iq.mat does not exist. Run cdma_duc_model to create this file.');
end

input_quant =quantizer('fixed','floor', 'wrap',[16,15]);
composite_iq_quant = quantize(input_quant,composite_iq);

%****************************************************************
% DDS Modeling 
% Note:  This DDS model does include phase quantization but not
%        phase dithering or Taylor modeling.  The DDS core output 
%        magnitude is, strangely, 0.5.
%****************************************************************

% Set up DDS output type parameters
NBITS_dds         = 18;
BINPT_dds         = 18;
PHASE_ACCUM_SIZE  = 17; % phase accum size; get from CoreGen GUI

% Create sinusoid scale factor
DDS_SCALE = (2^(NBITS_dds-1)-1) / 2^NBITS_dds;

% Calculate phase offset used by DDS
PHASE0_VAL = 2.5/2^NBITS_dds; % Found empirically
PHASE_OFFSET = asin(PHASE0_VAL/DDS_SCALE);

% Create quantizer objects
phase_quant=quantizer('fixed','nearest','wrap',[PHASE_ACCUM_SIZE+15,PHASE_ACCUM_SIZE]);
dds_quant  =quantizer('fixed','fix',  'wrap',[NBITS_dds,BINPT_dds]);
mult_quant =quantizer('fixed','floor','wrap',[36,34]);
sum_quant  =quantizer('fixed','floor','wrap',[18,17]);

% Make a column vector of the mixing frequencies (Hz) for each channel
f=Fc./Fs; 
wc=2.*pi.*f;

% Make a phase vector and quantize
ph = quantize(phase_quant, f * ([0:length(composite_iq)-1]+0));

% Make a matrix of the complex mixing sinusoids for each channel
sinusoid=exp(-j.* ((2.*pi.*ph) + PHASE_OFFSET)).*DDS_SCALE;

% Transpose the sinusoid matrix so the sinusoids are in columns
sinusoid = sinusoid.';

% Round sinusoid to replicate truncated values in lookup table
sinusoid=quantize(dds_quant, sinusoid);

%****************************************************************
% Complex Mixing
%****************************************************************

%------------------------------------------------------------------
% Zero-Padding:
% The DDS in the mixer takes many cycles to start up; meanwhile the
% input data has already made its way to the mixer.  This causes a 
% misalignment which we wish to model here.  We could just reset 
% the DDS when necessary, but using the reset line on the DDS 
% causes a host of timing issues and it's better to make good 
% hardware and then  model that instead of making poor hardware so 
% it matches the  model.  We can observe with which data input
% sample (on i_in_reg and q_in_reg in the mixer) the first DDS
% output for each channel (on dds_cos in the mixer) is aligned.  
% These values are put into the zpad matrix.  For example, if
% you see that the first channel 0 output on dds_cos lines up with
% the fourth value on i_in_reg then the first value of zpad is 4.
% By 'fourth value' I mean the fourth element in the 
% composite_iq_quant matrix.  This value can change if the DDS
% latency changes.
%------------------------------------------------------------------
zpad = [3 4 4]; % Hand-adjusted zero-padding used to make the model
                % line up with the hardware

% Replicate input for each channel
composite_iq_quant_shift ...
    = [[composite_iq_quant(zpad(1):end);zeros(zpad(1)-1,1)],...
       [composite_iq_quant(zpad(2):end);zeros(zpad(2)-1,1)],...
       [composite_iq_quant(zpad(3):end);zeros(zpad(3)-1,1)]];

% Mutiply the iq data matrix by the sinusoid matrix
mixed_iq=composite_iq_quant_shift.*sinusoid;

% Create mixer quantizer
mixer_quant =quantizer('fixed','fix','wrap',[17,16]);

% Quantize it
mixed_iq=quantize(mixer_quant,mixed_iq);

%****************************************************************
% Get Filter Coefficients
%****************************************************************
[h1_ddc, h2_ddc, h3_ddc] = cdma_ddc_filter(Quantize, Nfft, ShowPlots);

% Create filter quantizer object
filter_quant=quantizer('fixed','floor','wrap',[17,16]);

y1=[];y2=[];x=[];

%****************************************************************
% First FIR / Quantization Operation; 
% each column is filtered separately.
% This filter decimates by five, and the decimation phase is 
% arbitrary and may be any one of five.  To make sure this
% model matches the SysGen model, we set the Y1_PHASE value to
% the phase that matches the SysGen Fir Compiler filter.  This
% must be found by observing the SysGen simulation.  After it
% is set, we filter using 1 as the decimation factor and then
% decimate the resultant matrix manually using the Y1_PHASE
% value.  This value may need to be changed if the filter 
% parameters are modified.
%****************************************************************
Y1_PHASE=4; % see comment above

y1 = upfirdn(mixed_iq, h1_ddc, 1, 1); 
y1 = y1(Y1_PHASE:5:end,:); % Decimate
y1_quant=quantize(filter_quant,y1);

%****************************************************************
% Second FIR / Quantization Operation; 
% each column is filtered separately.
% To set Y2_PHASE, see coments for Y1_PHASE above
%****************************************************************
Y2_PHASE=4;
y2 = upfirdn(y1_quant, h2_ddc, 1, 1);  
y2 = y2(Y2_PHASE:5:end,:); % decimate
y2_quant=quantize(filter_quant,y2);

%****************************************************************
% Third FIR / Quantization Operation; 
% each column is filtered separately
% This filter doesn't decimate so no phase matching is required.
%****************************************************************
x = upfirdn(y2_quant, h3_ddc, 1, 1);  
x_quant=quantize(filter_quant,x);

%****************************************************************
% call ddc performance metrics calculation and plotting function
%****************************************************************
ddc_performance_metrics;
