%*******************************************************************************
% DISCLAIMER OF LIABILITY
%
% This text/file contains proprietary, confidential
% information of Xilinx, Inc., is distributed under license
% from Xilinx, Inc., and may be used, copied and/or
% disclosed only pursuant to the terms of a valid license
% agreement with Xilinx, Inc. Xilinx hereby grants you a 
% license to use this text/file solely for design, simulation, 
% implementation and creation of design files limited 
% to Xilinx devices or technologies. Use with non-Xilinx 
% devices or technologies is expressly prohibited and 
% immediately terminates your license unless covered by
% a separate agreement.
%
% Xilinx is providing this design, code, or information 
% "as-is" solely for use in developing programs and 
% solutions for Xilinx devices, with no obligation on the 
% part of Xilinx to provide support. By providing this design, 
% code, or information as one possible implementation of 
% this feature, application or standard, Xilinx is making no 
% representation that this implementation is free from any 
% claims of infringement. You are responsible for 
% obtaining any rights you may require for your implementation. 
% Xilinx expressly disclaims any warranty whatsoever with 
% respect to the adequacy of the implementation, including 
% but not limited to any warranties or representations that this
% implementation is free from claims of infringement, implied 
% warranties of merchantability or fitness for a particular 
% purpose.
%
% Xilinx products are not intended for use in life support
% appliances, devices, or systems. Use in such applications is
% expressly prohibited.
%
% Any modifications that are made to the Source Code are 
% done at the users sole risk and will be unsupported.
%
%
% Copyright (c) 2007 Xilinx, Inc. All rights reserved.
%
% This copyright and support notice must be retained as part 
% of this text at all times. 
%
%*******************************************************************************

%*******************************************************************************
% cdma_ddc_model.m
% 
% This is a model for the CDMA 3-channel DDC (Digital DownConverter).
% The model attempts to replicate the hardware as much as possible, by 
% quantizing appropriately.
% 
% Original:  Helen Tarn, Xilinx
% Update:    Kevin Neilson, Xilinx
%*******************************************************************************

%****************************************************************
% Set parameters
%****************************************************************
Quantize = 1;   % double (0) or quantized (1) coefficients and data 
Nfft = 1024;    % FFT size for spectral plot
ShowPlots = 0;  % Show filter impluse response plots
NUM_CH = 3;     % Define number of channels


Fc = [-1.25;0;1.25]; % Column vector of center mixing frequencies (MHz)
Fs = 61.44;          % Sampling Frequency
t = [0:1:999];       % Time vector

coding_gain = 10*log10(1.2288e6/3.84e3);  % coding gain = chip rate/baseband data rate

%****************************************************************
% Load complex input vector
%****************************************************************
%[ddc_in, x_ref, N] = cdma_ddc_input;  % acs_test_input
%[ddc_in, x_ref, N] = single_tone_test1; %single_tone_test2, single_tone_test3, intermodulation_test;
if (exist('composite_iq.mat'))
  load('composite_iq.mat');
else
  error('File composite_iq.mat does not exist. Run cdma_duc_model to create this file.');
end

%composite_iq = 2.*rand(10000,1) + j.*2.*rand(10000,1);


%****************************************************************
% DDS Modeling 
% Note:  This DDS model doesn't include phase quantization or
%        phase dithering.  The DDS core output magnitude is,
%        strangely, 0.5.
%****************************************************************

% Set up DDS output type parameters
NBITS_dds = 18;
BINPT_dds = 18;

% Create sinusoid scale factor
DDS_SCALE = (2^(NBITS_dds-1)-1) / 2^NBITS_dds;

% Calculate phase offset used by DDS
PHASE0_VAL = 2/2^NBITS_dds; % Found empirically
PHASE_OFFSET = asin(PHASE0_VAL/DDS_SCALE);

% Create quantizer objects
dds_quant =quantizer('fixed','floor','wrap',[NBITS_dds,BINPT_dds]);
mult_quant=quantizer('fixed','floor','wrap',[36,34]);
sum_quant =quantizer('fixed','floor','wrap',[18,17]);

% Make a column vector of the mixing frequencies (rads) for each channel
wc=2.*pi.*Fc./Fs;% 

% Make a matrix of the complex mixing sinusoids for each channel
sinusoid=exp(-j.* (wc * [0:length(composite_iq)-1] + PHASE_OFFSET)).*DDS_SCALE;

% Transpose the sinusoid matrix so the sinusoids are in columns
sinusoid = sinusoid.';

% Round sinusoid to replicate truncated values in lookup table
sinusoid=quantize(dds_quant, sinusoid);

%****************************************************************
% Complex Mixing
%****************************************************************

zpad = [4 4 5]; % Hand-adjusted zero-padding used to make the model
                 % line up with the hardware

% Replicate input for each channel
composite_iq = [[composite_iq(zpad(1):end);zeros(zpad(1)-1,1)],...
                [composite_iq(zpad(2):end);zeros(zpad(2)-1,1)],...
                [composite_iq(zpad(3):end);zeros(zpad(3)-1,1)]];

% Mutiply the iq data matrix by the sinusoid matrix
mixed_iq=composite_iq.*sinusoid;

% Quantize it
mixed_iq=quantize(mult_quant,mixed_iq);

mixed_iq(1,:) = repmat(real(mixed_iq(1,1)).*(1+j),[1,3]);%%%


%****************************************************************
% Get Filter Coefficients
%****************************************************************
[h1_ddc, h2_ddc, h3_ddc] = cdma_ddc_filter(Quantize, Nfft, ShowPlots);

% Create filter quantizer object
filter_quant=quantizer('fixed','floor','wrap',[17,16]);

y1=[];y2=[];x=[];

%****************************************************************
% First FIR / Quantization Operation; 
% each column is filtered separately.
% To make the 'upfirdn' function match the functionality of the
% SysGen FIR 5x decimator, I have to pad the input with two
% zeros and then delete the first output.
%****************************************************************
fir_zpad = zeros(2,3);
y1 = upfirdn([fir_zpad;mixed_iq], h1_ddc, 1, 5); 
y1 = y1(2:end,:);
y1_quant=quantize(filter_quant,y1);

%****************************************************************
% Second FIR / Quantization Operation; 
% each column is filtered separately.
% To make the 'upfirdn' function match the functionality of the
% SysGen FIR 5x decimator, I have to pad the input with two
% zeros and then delete the first output.
%****************************************************************
y2 = upfirdn([fir_zpad;y1_quant], h2_ddc, 1, 5);  
y2 = y2(2:end,:);
y2_quant=quantize(filter_quant,y2);

%****************************************************************
% Third FIR / Quantization Operation; 
% each column is filtered separately
%****************************************************************
x = upfirdn(y2_quant, h3_ddc, 1, 1);  
x_quant=quantize(filter_quant,x);

%****************************************************************
% call ddc performance metrics calculation and plotting function
%****************************************************************
ddc_performance_metrics;