%*******************************************************************************
% DISCLAIMER OF LIABILITY
%
% This text/file contains proprietary, confidential
% information of Xilinx, Inc., is distributed under license
% from Xilinx, Inc., and may be used, copied and/or
% disclosed only pursuant to the terms of a valid license
% agreement with Xilinx, Inc. Xilinx hereby grants you a 
% license to use this text/file solely for design, simulation, 
% implementation and creation of design files limited 
% to Xilinx devices or technologies. Use with non-Xilinx 
% devices or technologies is expressly prohibited and 
% immediately terminates your license unless covered by
% a separate agreement.
%
% Xilinx is providing this design, code, or information 
% "as-is" solely for use in developing programs and 
% solutions for Xilinx devices, with no obligation on the 
% part of Xilinx to provide support. By providing this design, 
% code, or information as one possible implementation of 
% this feature, application or standard, Xilinx is making no 
% representation that this implementation is free from any 
% claims of infringement. You are responsible for 
% obtaining any rights you may require for your implementation. 
% Xilinx expressly disclaims any warranty whatsoever with 
% respect to the adequacy of the implementation, including 
% but not limited to any warranties or representations that this
% implementation is free from claims of infringement, implied 
% warranties of merchantability or fitness for a particular 
% purpose.
%
% Xilinx products are not intended for use in life support
% appliances, devices, or systems. Use in such applications is
% expressly prohibited.
%
% Any modifications that are made to the Source Code are 
% done at the users sole risk and will be unsupported.
%
%
% Copyright (c) 2007 Xilinx, Inc. All rights reserved.
%
% This copyright and support notice must be retained as part 
% of this text at all times. 
%
%*******************************************************************************

%*******************************************************************************
% cdma_duc_model.m
% 
% This is a model for the CDMA 3-channel DUC (Digital UpConverter).
% The model attempts to replicate the hardware as much as possible, by 
% quantizing appropriately.
% 
% Original:  Helen Tarn, Xilinx
% Update:    Kevin Neilson, Xilinx
%*******************************************************************************

%****************************************************************
% Set parameters
%****************************************************************
Quantize = 1;   % double (0) or quantized (1) coefficients and data 
Nfft = 8192;    % FFT size for spectral plot
ShowPlots = 0;  % Show filter impluse response plots
NUM_CH = 3;     % Define number of channels


Fc = [-1.25;0;1.25];   % Column vector of center mixing frequencies (MHz)
scale_vector=[1 1 1];  % Scale factors for each channel [0:1]
Fs = 61.44;            % Sampling Frequency (MHz)
t = [0:1:999];         % Time vector

% data_type may be 'random' (quasi-Gaussian), 
% 'sinusoid' (a different frequency of sinusoid for each channel),
% or 'lemniscate' (a lemniscate with different # of lobes for each channel)
data_type='random';

%****************************************************************
% Generate complex input matrix
% The input matrix has a column for each channel.
%****************************************************************
iq_in=gen_input(data_type,NUM_CH,length(t),16);

%****************************************************************
% Get Filter Coefficients
%****************************************************************
[h, h1_duc, h2_duc, h3_duc, h_duc] = cdma_duc_filter(Quantize, Nfft, ShowPlots);

% Create filter quantizer object
filter_quant1=quantizer('fixed','floor','wrap',[17,16]);
filter_quant2=quantizer('fixed','floor','wrap',[18,17]);

y0=[];y1=[];y2=[];

%****************************************************************
% First FIR / Quantization Operation; 
% each column is filtered separately
%****************************************************************
y0 = upfirdn(iq_in, h1_duc, 2);  
y0_quant=quantize(filter_quant1,y0);

%****************************************************************
% Scaling Operation
% each column is scaled separately
%****************************************************************
scale_matrix=repmat(scale_vector,[NUM_CH,1]).*eye(NUM_CH);
y0_quant = y0_quant*scale_matrix;

%****************************************************************
% Second FIR / Quantization Operation; 
% each column is filtered separately
%****************************************************************
y1 = upfirdn(y0_quant, h2_duc, 5);  
y1_quant=quantize(filter_quant1,y1);

%****************************************************************
% Third FIR / Quantization Operation; 
% each column is filtered separately
%****************************************************************
y2 = upfirdn(y1_quant, h3_duc, 5);  
y2_quant=quantize(filter_quant2,y2);

%*******************************************************************************
% DDS Modeling 
% Note:  This DDS model does include phase quantization but not
%        phase dithering or Taylor modeling
%
% DDS Offset:
% A certain number of samples=DDS_OFFSET must be dropped from the
% beginning of the sinusoid signals for latency matching.  This is
% because the DDS starts up before the DUC input data passes 
% through all the filter latency, so DDS_OFFSET samples (per chan)
% are dropped to match the hardware more closely.  The DDS could be
% held in reset until required, but this requires extra hardware 
% with a lot of critical paths.  The DDS_OFFSET value is dependent
% upon all the filter latencies and other design-dependent parameters.
% This value is (T1-T2)/NUM_CH, where T1 is the time at which 
% align_vout in the mixer goes high, and T2 is the time at which
% the first DDS output is valid.
% That is the general formula.  However, there is another source
% of latency. The SysGen FIR filter doesn't operate exactly like the 
% Matlab function 'upfirdn'.  That is, the SysGen FIR will, in many 
% configurations, assert  the 'valid_out' signal before the first valid
% output.  That is, some  zeros are prepended to the output. This latency
% must be incorporated into DDS_OFFSET.  The total value of DDS_OFFSET 
% is basically the number of cycles (per channel) between the
% assertion of align_vout in the SysGen mixer and the appearance of
% the first valid output on i_align.  This can be difficult to find since
% at the beginning of the simulation the valid filter outputs are zero.
% So the easiest way to calculate this value is this:
%
% DDS_OFFSET = (T_nz_i_align0 - T2)/(2*NUM_CH) - Index_nz_y2_0 + 1
%
% where
%    T_nz_i_align0 is the time of the start of the first nonzero output
%         of i_align channel 0 (real).
%    T2 is the time of the start of the first valid channel 0 DDS output.
%    Index_nz_y2_0 is the index of the first row of y2_quant that has a
%         nonzero output for the real part of channel 0 (column 1).
%    2*NUM_CH is the number of clock cycles required to generate
%         multiplexed i and q data for each channel
%
% By finding the first nonzero outputs of the real part of channel 0 of
% y2_quant (in the Matlab model) and i_align (the equivalent in the 
% Sysgen model) a correlation can be done.  Then the latency of the 
% DDS is factored in (using T2) and the number of channels is accounted
% for, and the final offset ensures that the alignment of the DDS
% and the third filter's output are the same in both the Matlab and
% SysGen models.
% This could all be automated if i_align and dds_cos were written to the
% workspace.  That is something to think about for the future.
%*******************************************************************************

DDS_OFFSET = 188;

% Set up DDS output type parameters
NBITS_dds         = 18;
BINPT_dds         = 18;
PHASE_ACCUM_SIZE  = 17; % phase accum size; get from CoreGen GUI

% Create sinusoid scale factor
DDS_SCALE = (2^(NBITS_dds-1)-1) / 2^NBITS_dds;

% Calculate phase offset used by DDS
PHASE0_VAL = 0/2^NBITS_dds; % Found empirically
PHASE_OFFSET = asin(PHASE0_VAL/DDS_SCALE);

% Create quantizer objects
phase_quant=quantizer('fixed','nearest','wrap',[PHASE_ACCUM_SIZE+15,PHASE_ACCUM_SIZE]);
dds_quant  =quantizer('fixed','fix',  'wrap',[NBITS_dds,BINPT_dds]);
mult_quant =quantizer('fixed','floor','wrap',[36,34]);
sum_quant  =quantizer('fixed','floor','wrap',[18,17]);

% Make a column vector of the mixing frequencies (Hz) for each channel
f=Fc./Fs; 
wc=2.*pi.*f;

% Make a phase vector and quantize
ph = quantize(phase_quant, f * ([0:length(y2_quant)-1]+DDS_OFFSET));

% Make a matrix of the complex mixing sinusoids for each channel
sinusoid=exp(j.* 2.*pi.*ph + PHASE_OFFSET).*DDS_SCALE;

% Transpose the sinusoid matrix so the sinusoids are in columns
sinusoid = sinusoid.';

% Round sinusoid to replicate truncated values in lookup table
sinusoid=quantize(dds_quant, sinusoid);

%****************************************************************
% Complex Mixing
%****************************************************************

% Mutiply the iq data matrix by the sinusoid matrix
mixed_iq=y2_quant.*sinusoid;

% Quantize it
mixed_iq=quantize(mult_quant,mixed_iq);

% Sum the rows of the channel columns to get the composite iq
composite_iq=sum(mixed_iq,2);

% Quantize it
composite_iq=quantize(sum_quant,composite_iq);

%****************************************************************
% call performance metrics calculation and plotting function
%****************************************************************
performance_metrics;

%****************************************************************
% Save composite_iq to file for use with DDC model
%****************************************************************

% Before we save composite_iq, we scale it up to emulate the
% action of an automatic gain control.
composite_iq = 0.99.*composite_iq./max(abs(composite_iq));

% Save it to a file for use with the DDC model
save('composite_iq.mat','composite_iq');
if (exist('../ddc'))
  save('../ddc/composite_iq.mat','composite_iq');
end
